@@ -16,14 +16,14 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
+ * Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
- 
+
 /*
- * Modified by the gedit Team, 2006. See the AUTHORS file for a 
- * list of people on the gedit Team.  
- * See the ChangeLog files for a list of changes. 
+ * Modified by the gedit Team, 2006. See the AUTHORS file for a
+ * list of people on the gedit Team.
+ * See the ChangeLog files for a list of changes.
  *
  * $Id$
  */
@@ -35,14 +35,14 @@
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 
 #include "gedit-history-entry.h"
 
 enum {
 	PROP_0,
 	PROP_HISTORY_ID,
-	PROP_HISTORY_LENGTH
+	PROP_HISTORY_LENGTH,
+	PROP_ENABLE_COMPLETION
 };
 
 #define MIN_ITEM_LEN 3
@@ -57,13 +57,13 @@ struct _GeditHistoryEntryPrivate
 {
 	gchar              *history_id;
 	guint               history_length;
-	
+
 	GtkEntryCompletion *completion;
-	
-	GConfClient        *gconf_client;
+
+	GSettings          *settings;
 };
 
-G_DEFINE_TYPE(GeditHistoryEntry, gedit_history_entry, GTK_TYPE_COMBO_BOX_ENTRY)
+G_DEFINE_TYPE(GeditHistoryEntry, gedit_history_entry, GTK_TYPE_COMBO_BOX_TEXT)
 
 static void
 gedit_history_entry_set_property (GObject      *object,
@@ -77,16 +77,21 @@ gedit_history_entry_set_property (GObject      *object,
 
 	entry = GEDIT_HISTORY_ENTRY (object);
 
-	switch (prop_id) {
-	case PROP_HISTORY_ID:
-		entry->priv->history_id = g_value_dup_string (value);
-		break;
-	case PROP_HISTORY_LENGTH:
-		gedit_history_entry_set_history_length (entry,
-						     g_value_get_uint (value));
-		break;
-	default:
-		break;
+	switch (prop_id)
+	{
+		case PROP_HISTORY_ID:
+			entry->priv->history_id = g_value_dup_string (value);
+			break;
+		case PROP_HISTORY_LENGTH:
+			gedit_history_entry_set_history_length (entry,
+								g_value_get_uint (value));
+			break;
+		case PROP_ENABLE_COMPLETION:
+			gedit_history_entry_set_enable_completion (entry,
+			                                           g_value_get_boolean (value));
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, spec);
 	}
 }
 
@@ -102,25 +107,39 @@ gedit_history_entry_get_property (GObject    *object,
 
 	priv = GEDIT_HISTORY_ENTRY (object)->priv;
 
-	switch (prop_id) {
-	case PROP_HISTORY_ID:
-		g_value_set_string (value, priv->history_id);
-		break;
-	case PROP_HISTORY_LENGTH:
-		g_value_set_uint (value, priv->history_length);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, spec);
+	switch (prop_id)
+	{
+		case PROP_HISTORY_ID:
+			g_value_set_string (value, priv->history_id);
+			break;
+		case PROP_HISTORY_LENGTH:
+			g_value_set_uint (value, priv->history_length);
+			break;
+		case PROP_ENABLE_COMPLETION:
+			g_value_set_boolean (value, gedit_history_entry_get_enable_completion (GEDIT_HISTORY_ENTRY (object)));
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, spec);
 	}
 }
 
 static void
-gedit_history_entry_destroy (GtkObject *object)
+gedit_history_entry_dispose (GObject *object)
 {
+	GeditHistoryEntryPrivate *priv;
+
+	priv = GEDIT_HISTORY_ENTRY (object)->priv;
+
 	gedit_history_entry_set_enable_completion (GEDIT_HISTORY_ENTRY (object),
 						   FALSE);
 
-	GTK_OBJECT_CLASS (gedit_history_entry_parent_class)->destroy (object);
+	if (priv->settings != NULL)
+	{
+		g_object_unref (G_OBJECT (priv->settings));
+		priv->settings = NULL;
+	}
+
+	G_OBJECT_CLASS (gedit_history_entry_parent_class)->dispose (object);
 }
 
 static void
@@ -129,29 +148,58 @@ gedit_history_entry_finalize (GObject *object)
 	GeditHistoryEntryPrivate *priv;
 
 	priv = GEDIT_HISTORY_ENTRY (object)->priv;
-	
-	g_free (priv->history_id);
 
-	if (priv->gconf_client != NULL)
-	{
-		g_object_unref (G_OBJECT (priv->gconf_client));
-		priv->gconf_client = NULL;
-	}
+	g_free (priv->history_id);
 
 	G_OBJECT_CLASS (gedit_history_entry_parent_class)->finalize (object);
 }
 
-static void 
+static void
+gedit_history_entry_load_history (GeditHistoryEntry *entry)
+{
+	gchar **items;
+	gsize i;
+
+	items = g_settings_get_strv (entry->priv->settings,
+				     entry->priv->history_id);
+	i = 0;
+
+	gtk_combo_box_text_remove_all (GTK_COMBO_BOX_TEXT (entry));
+
+	/* Now the default value is an empty string so we have to take care
+	   of it to not add the empty string in the search list */
+	while (items[i] != NULL && *items[i] != '\0' &&
+	       i < entry->priv->history_length)
+	{
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (entry), items[i]);
+		i++;
+	}
+
+	g_strfreev (items);
+}
+
+static void
+gedit_history_entry_constructed (GObject *object)
+{
+	/* We must load the history after the object has been constructed,
+	 * to ensure that the model is set properly.
+	 */
+	gedit_history_entry_load_history (GEDIT_HISTORY_ENTRY (object));
+
+	G_OBJECT_CLASS (gedit_history_entry_parent_class)->constructed (object);
+}
+
+static void
 gedit_history_entry_class_init (GeditHistoryEntryClass *klass)
 {
 	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
-	GtkObjectClass *gtkobject_class = GTK_OBJECT_CLASS (klass);
-	
+
 	object_class->set_property = gedit_history_entry_set_property;
 	object_class->get_property = gedit_history_entry_get_property;
+	object_class->dispose = gedit_history_entry_dispose;
 	object_class->finalize = gedit_history_entry_finalize;
-	gtkobject_class->destroy = gedit_history_entry_destroy;
-	
+	object_class->constructed = gedit_history_entry_constructed;
+
 	g_object_class_install_property (object_class,
 					 PROP_HISTORY_ID,
 					 g_param_spec_string ("history-id",
@@ -159,6 +207,7 @@ gedit_history_entry_class_init (GeditHistoryEntryClass *klass)
 							      "History ID",
 							      NULL,
 							      G_PARAM_READWRITE |
+							      G_PARAM_CONSTRUCT_ONLY |
 							      G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (object_class,
@@ -172,9 +221,16 @@ gedit_history_entry_class_init (GeditHistoryEntryClass *klass)
 							    G_PARAM_READWRITE |
 							    G_PARAM_STATIC_STRINGS));
 
-	/* TODO: Add enable-completion property */
+	g_object_class_install_property (object_class,
+	                                 PROP_ENABLE_COMPLETION,
+	                                 g_param_spec_boolean ("enable-completion",
+	                                                       "Enable Completion",
+	                                                       "Wether the completion is enabled",
+	                                                       TRUE,
+	                                                       G_PARAM_READWRITE |
+	                                                       G_PARAM_STATIC_STRINGS));
 
-	g_type_class_add_private (object_class, sizeof(GeditHistoryEntryPrivate));
+	g_type_class_add_private (object_class, sizeof (GeditHistoryEntryPrivate));
 }
 
 static GtkListStore *
@@ -188,42 +244,25 @@ get_history_store (GeditHistoryEntry *entry)
 	return (GtkListStore *) store;
 }
 
-static char *
-get_history_key (GeditHistoryEntry *entry)
-{
-	gchar *tmp;
-	gchar *key;
-
-	/*
-	 * We store the data under /apps/gnome-settings/
-	 * like the old GnomeEntry did. Maybe we should
-	 * consider moving it to the /gedit GConf prefix...
-	 * Or maybe we should just switch away from GConf.
-	 */
-
-	tmp = gconf_escape_key (entry->priv->history_id, -1);
-	key = g_strconcat ("/apps/gnome-settings/",
-			   "gedit",
-			   "/history-",
-			   tmp,
-			   NULL);
-	g_free (tmp);
-
-	return key;
-}
-
-static GSList *
-get_history_list (GeditHistoryEntry *entry)
+static gchar **
+get_history_items (GeditHistoryEntry *entry)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
+	GPtrArray *array;
 	gboolean valid;
-	GSList *list = NULL;
+	gint n_children;
+	gint text_column;
 
 	store = get_history_store (entry);
+	text_column = gtk_combo_box_get_entry_text_column (GTK_COMBO_BOX (entry));
 
 	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store),
 					       &iter);
+	n_children = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store),
+						     NULL);
+
+	array = g_ptr_array_sized_new (n_children + 1);
 
 	while (valid)
 	{
@@ -231,48 +270,49 @@ get_history_list (GeditHistoryEntry *entry)
 
 		gtk_tree_model_get (GTK_TREE_MODEL (store),
 				    &iter,
-				    0, &str,
+				    text_column, &str,
 				    -1);
 
-		list = g_slist_prepend (list, str);
+		g_ptr_array_add (array, str);
 
 		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (store),
 						  &iter);
 	}
 
-	return g_slist_reverse (list);
+	g_ptr_array_add (array, NULL);
+
+	return (gchar **)g_ptr_array_free (array, FALSE);
 }
 
 static void
 gedit_history_entry_save_history (GeditHistoryEntry *entry)
 {
-	GSList *gconf_items;
-	gchar *key;
+	gchar **items;
 
 	g_return_if_fail (GEDIT_IS_HISTORY_ENTRY (entry));
 
-	gconf_items = get_history_list (entry);
-	key = get_history_key (entry);
+	items = get_history_items (entry);
 
-	gconf_client_set_list (entry->priv->gconf_client,
-			      key,
-			      GCONF_VALUE_STRING,
-			      gconf_items,
-			      NULL);
+	g_settings_set_strv (entry->priv->settings,
+			     entry->priv->history_id,
+			     (const gchar * const *)items);
 
-	g_slist_foreach (gconf_items, (GFunc) g_free, NULL);
-	g_slist_free (gconf_items);
-	g_free (key);
+	g_strfreev (items);
 }
 
 static gboolean
-remove_item (GtkListStore *store,
-	     const gchar  *text)
+remove_item (GeditHistoryEntry *entry,
+	     const gchar       *text)
 {
+	GtkListStore *store;
 	GtkTreeIter iter;
+	gint text_column;
 
 	g_return_val_if_fail (text != NULL, FALSE);
 
+	store = get_history_store (entry);
+	text_column = gtk_combo_box_get_entry_text_column (GTK_COMBO_BOX (entry));
+
 	if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter))
 		return FALSE;
 
@@ -282,7 +322,7 @@ remove_item (GtkListStore *store,
 
 		gtk_tree_model_get (GTK_TREE_MODEL (store),
 				    &iter,
-				    0,
+				    text_column,
 				    &item_text,
 				    -1);
 
@@ -313,7 +353,7 @@ clamp_list_store (GtkListStore *store,
 
 	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path))
 	{
-		while (1)
+		while (TRUE)
 		{
 			if (!gtk_list_store_remove (store, &iter))
 				break;
@@ -329,11 +369,10 @@ insert_history_item (GeditHistoryEntry *entry,
 		     gboolean           prepend)
 {
 	GtkListStore *store;
-	GtkTreeIter iter;
 
 	if (g_utf8_strlen (text, -1) <= MIN_ITEM_LEN)
 		return;
-		
+
 	store = get_history_store (entry);
 
 	/* remove the text from the store if it was already
@@ -341,20 +380,14 @@ insert_history_item (GeditHistoryEntry *entry,
 	 * before inserting the new row, otherwise appending
 	 * would not work */
 
-	if (!remove_item (store, text))
+	if (!remove_item (entry, text))
 		clamp_list_store (store,
 				  entry->priv->history_length - 1);
 
 	if (prepend)
-		gtk_list_store_insert (store, &iter, 0);
+		gtk_combo_box_text_prepend_text (GTK_COMBO_BOX_TEXT (entry), text);
 	else
-		gtk_list_store_append (store, &iter);
-
-	gtk_list_store_set (store,
-			    &iter,
-			    0,
-			    text,
-			    -1);
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (entry), text);
 
 	gedit_history_entry_save_history (entry);
 }
@@ -379,53 +412,12 @@ gedit_history_entry_append_text (GeditHistoryEntry *entry,
 	insert_history_item (entry, text, FALSE);
 }
 
-static void
-gedit_history_entry_load_history (GeditHistoryEntry *entry)
-{
-	GSList *gconf_items, *l;
-	GtkListStore *store;
-	GtkTreeIter iter;
-	gchar *key;
-	guint i;
-
-	g_return_if_fail (GEDIT_IS_HISTORY_ENTRY (entry));
-
-	store = get_history_store (entry);
-	key = get_history_key (entry);
-
-	gconf_items = gconf_client_get_list (entry->priv->gconf_client,
-					     key,
-					     GCONF_VALUE_STRING,
-					     NULL);
-
-	gtk_list_store_clear (store);
-
-	for (l = gconf_items, i = 0;
-	     l != NULL && i < entry->priv->history_length;
-	     l = l->next, i++)
-	{
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, 
-				    &iter,
-				    0,
-				    l->data,
-				    -1);
-	}
-
-	g_slist_foreach (gconf_items, (GFunc) g_free, NULL);
-	g_slist_free (gconf_items);
-	g_free (key);
-}
-
 void
 gedit_history_entry_clear (GeditHistoryEntry *entry)
 {
-	GtkListStore *store;
-
 	g_return_if_fail (GEDIT_IS_HISTORY_ENTRY (entry));
 
-	store = get_history_store (entry);
-	gtk_list_store_clear (store);
+	gtk_combo_box_text_remove_all (GTK_COMBO_BOX_TEXT (entry));
 
 	gedit_history_entry_save_history (entry);
 }
@@ -442,8 +434,8 @@ gedit_history_entry_init (GeditHistoryEntry *entry)
 	priv->history_length = GEDIT_HISTORY_ENTRY_HISTORY_LENGTH_DEFAULT;
 
 	priv->completion = NULL;
-	
-	priv->gconf_client = gconf_client_get_default ();
+
+	priv->settings = g_settings_new ("org.gnome.gedit.state.history-entry");
 }
 
 void
@@ -466,29 +458,21 @@ gedit_history_entry_get_history_length (GeditHistoryEntry *entry)
 	return entry->priv->history_length;
 }
 
-gchar *
-gedit_history_entry_get_history_id (GeditHistoryEntry *entry)
-{
-	g_return_val_if_fail (GEDIT_IS_HISTORY_ENTRY (entry), NULL);
-
-	return g_strdup (entry->priv->history_id);
-}
-
 void
 gedit_history_entry_set_enable_completion (GeditHistoryEntry *entry,
 					   gboolean           enable)
 {
 	g_return_if_fail (GEDIT_IS_HISTORY_ENTRY (entry));
-	
+
 	if (enable)
 	{
 		if (entry->priv->completion != NULL)
 			return;
-		
+
 		entry->priv->completion = gtk_entry_completion_new ();
-		gtk_entry_completion_set_model (entry->priv->completion, 
+		gtk_entry_completion_set_model (entry->priv->completion,
 						GTK_TREE_MODEL (get_history_store (entry)));
-		
+
 		/* Use model column 0 as the text column */
 		gtk_entry_completion_set_text_column (entry->priv->completion, 0);
 
@@ -497,9 +481,9 @@ gedit_history_entry_set_enable_completion (GeditHistoryEntry *entry,
 
 		gtk_entry_completion_set_popup_completion (entry->priv->completion, FALSE);
 		gtk_entry_completion_set_inline_completion (entry->priv->completion, TRUE);
-	
+
 		/* Assign the completion to the entry */
-		gtk_entry_set_completion (GTK_ENTRY (gedit_history_entry_get_entry(entry)), 
+		gtk_entry_set_completion (GTK_ENTRY (gedit_history_entry_get_entry (entry)),
 					  entry->priv->completion);
 	}
 	else
@@ -507,20 +491,20 @@ gedit_history_entry_set_enable_completion (GeditHistoryEntry *entry,
 		if (entry->priv->completion == NULL)
 			return;
 
-		gtk_entry_set_completion (GTK_ENTRY (gedit_history_entry_get_entry (entry)), 
+		gtk_entry_set_completion (GTK_ENTRY (gedit_history_entry_get_entry (entry)),
 					  NULL);
-		
+
 		g_object_unref (entry->priv->completion);
-		
+
 		entry->priv->completion = NULL;
 	}
 }
-							 
+
 gboolean
 gedit_history_entry_get_enable_completion (GeditHistoryEntry *entry)
 {
 	g_return_val_if_fail (GEDIT_IS_HISTORY_ENTRY (entry), FALSE);
-	
+
 	return entry->priv->completion != NULL;
 }
 
@@ -528,50 +512,23 @@ GtkWidget *
 gedit_history_entry_new (const gchar *history_id,
 			 gboolean     enable_completion)
 {
-	GtkWidget *ret;
-	GtkListStore *store;
-
 	g_return_val_if_fail (history_id != NULL, NULL);
 
-	/* Note that we are setting the model, so
-	 * user must be careful to always manipulate
-	 * data in the history through gedit_history_entry_
-	 * functions.
-	 */
+	enable_completion = (enable_completion != FALSE);
 
-	store = gtk_list_store_new (1, G_TYPE_STRING);
-
-	ret = g_object_new (GEDIT_TYPE_HISTORY_ENTRY,
-			    "history-id", history_id,
-	                    "model", store,
-			    "text-column", 0,
-	                    NULL);
-
-	g_object_unref (store);
-
-	/* loading has to happen after the model
-	 * has been set. However the model is not a 
-	 * G_PARAM_CONSTRUCT property of GtkComboBox
-	 * so we cannot do this in the constructor.
-	 * For now we simply do here since this widget is 
-	 * not bound to other programming languages.
-	 * A maybe better alternative is to override the 
-	 * model property of combobox and mark CONTRUCT_ONLY.
-	 * This would also ensure that the model cannot be
-	 * set explicitely at a later time.
-	 */
-	gedit_history_entry_load_history (GEDIT_HISTORY_ENTRY (ret));
-
-	gedit_history_entry_set_enable_completion (GEDIT_HISTORY_ENTRY (ret),
-						   enable_completion);
-						   
-	return ret;
+	return g_object_new (GEDIT_TYPE_HISTORY_ENTRY,
+	                     "has-entry", TRUE,
+	                     "entry-text-column", 0,
+	                     "id-column", 1,
+	                     "history-id", history_id,
+	                     "enable-completion", enable_completion,
+	                     NULL);
 }
 
 /*
  * Utility function to get the editable text entry internal widget.
- * I would prefer to not expose this implementation detail and 
- * simply make the GeditHistoryEntry widget implement the 
+ * I would prefer to not expose this implementation detail and
+ * simply make the GeditHistoryEntry widget implement the
  * GtkEditable interface. Unfortunately both GtkEditable and
  * GtkComboBox have a "changed" signal and I am not sure how to
  * handle the conflict.
@@ -616,18 +573,23 @@ gedit_history_entry_set_escape_func (GeditHistoryEntry           *entry,
 	g_return_if_fail (cells->data != NULL && cells->next == NULL);
 
 	if (escape_func != NULL)
+	{
 		gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (entry),
 						    GTK_CELL_RENDERER (cells->data),
 						    (GtkCellLayoutDataFunc) escape_cell_data_func,
 						    escape_func,
 						    NULL);
+	}
 	else
+	{
 		gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (entry),
 						    GTK_CELL_RENDERER (cells->data),
 						    NULL,
 						    NULL,
 						    NULL);
+	}
 
 	g_list_free (cells);
 }
-/* ex:ts=8:noet: */
+
+/* ex:set ts=8 noet: */
